require "em-synchrony/em-http"

module ApiTransformer
  # Process request blocks
  class BackendRequest
    attr_reader :name
    attr_accessor :path, :method, :query_params, :form_params, :json_params,
                  :cookie_params, :header_params

    def initialize(name, base_url, frontend_headers)
      @name = name
      @base_url = base_url
      @frontend_headers = frontend_headers

      @query_params = {}
      @form_params = {}
      @json_params = {}
      @cookie_params = {}
      @header_params = {}
    end

    def send
      url = @base_url + @path

      EM::HttpRequest.new(url).send(
        "a#{@method}",
        query: @query_params,
        body: body,
        head: headers
      )
    end

    private

    def body
      case [@form_params.any?, @json_params.any?]
      when [true, true]
        fail RequestError, "A request cannot have both json and form parameters"
      when [true, false]
        @form_params
      when [false, true]
        @json_params.to_json
      when [false, false]
        nil
      end
    end

    def headers
      ret = [
        stripped_frontend_headers,
        @header_params,
        content_type,
        cookies
      ].reduce(&:merge)

      ret.delete("Host")
      ret
    end

    def stripped_frontend_headers
      @frontend_headers.reject { |key, _| key == "Content-Length" }
    end

    def content_type
      if @json_params.any?
        { "Content-Type" => "application/json" }
      elsif @form_params.any?
        { "Content-Type" => "application/x-www-form-urlencoded" }
      else
        {}
      end
    end

    def cookies
      if @cookie_params.any?
        cookie_string = @cookie_params
          .map { |key, value| "#{key}=#{value};" }
          .join(" ")

        { "Cookie" => cookie_string }
      else
        {}
      end
    end
  end
end
