module ApiTransformer
  # Processes the request block
  class BackendRequestSender
    def initialize(name, options, block, frontend_headers, helper_blocks)
      @backend_request = BackendRequest.new(
        name,
        options[:base_url],
        frontend_headers
      )

      @block = block
      @helper_blocks = helper_blocks
    end

    def send(backend_responses)
      @helper_blocks.each { |block| instance_eval(&block) }
      instance_exec(backend_responses, &@block)

      unless @backend_request.method
        fail RequestError, "Missing method for backend request: #{request_name}"
      end

      @backend_request.send
    end

    def request_name
      @backend_request.name
    end

    private

    def path(value)
      @backend_request.path = value
    end

    def method(value)
      @backend_request.method = value
    end

    def query_param(key, value)
      @backend_request.query_params[key] = value
    end

    def form_param(key, value)
      @backend_request.form_params[key] = value
    end

    def json_param(key, value)
      @backend_request.json_params[key] = value
    end

    def cookie_param(key, value)
      @backend_request.cookie_params[key] = value
    end

    def header_param(key, value)
      @backend_request.header_params[key] = value
    end
  end
end
