require "json"

module ApiTransformer
  # Container for frontend response data
  class FrontendResponse
    attr_accessor :status
    attr_writer :body, :content_type

    def initialize
      @hash = {}
      @cookies = {}
      @headers = {}
    end

    def set(key, value)
      @hash[key] = value
    end

    def set_cookie(key, value)
      @cookies[key] = value
    end

    def set_header(key, value)
      @headers[key] = value
    end

    def body
      @hash.any? && @hash.to_json || @body
    end

    def headers
      [@headers, cookie_header, content_type_header].reduce(&:merge)
    end

    private

    def cookie_header
      if @cookies.any?
        pairs = @cookies.map { |key, value| "#{key}=#{value}" }
        { "Set-Cookie" => pairs.join("; ") }
      else
        {}
      end
    end

    def content_type_header
      { "Content-Type" => content_type }
    end

    def content_type
      if @content_type
        @content_type
      elsif @hash
        "application/json"
      else
        "text/plain"
      end
    end
  end
end
